@@ -37,19 +37,19 @@ class RepositoryTestCaseBase(BaseTestCase):
         self.repository.__enter__()
 
     def tearDown(self):
-        self.repository.close()
+        self.repository.__exit__(None, None, None)
         shutil.rmtree(self.tmppath)
 
     def reopen(self, exclusive=UNSPECIFIED):
-        if self.repository:
-            self.repository.close()
+        self.repository.__exit__(None, None, None)
         self.repository = self.open(exclusive=exclusive)
+        self.repository.__enter__()
 
     def add_keys(self):
         self.repository.put(H(0), b'foo')
         self.repository.put(H(1), b'bar')
         self.repository.put(H(3), b'bar')
-        self.repository.commit()
+        self.repository.commit(compact=False)
         self.repository.put(H(1), b'bar2')
         self.repository.put(H(2), b'boo')
         self.repository.delete(H(3))
@@ -74,7 +74,7 @@ class RepositoryTestCase(RepositoryTestCaseBase):
         self.assert_equal(self.repository.get(key50), b'SOMEDATA')
         self.repository.delete(key50)
         self.assert_raises(Repository.ObjectNotFound, lambda: self.repository.get(key50))
-        self.repository.commit()
+        self.repository.commit(compact=False)
         self.repository.close()
         with self.open() as repository2:
             self.assert_raises(Repository.ObjectNotFound, lambda: repository2.get(key50))
@@ -88,10 +88,10 @@ class RepositoryTestCase(RepositoryTestCaseBase):
         """
         self.repository.put(H(0), b'foo')
         self.repository.put(H(1), b'foo')
-        self.repository.commit()
+        self.repository.commit(compact=False)
         self.repository.delete(H(0))
         self.repository.put(H(1), b'bar')
-        self.repository.commit()
+        self.repository.commit(compact=False)
         self.assert_equal(self.repository.get(H(1)), b'bar')
 
     def test_consistency(self):
@@ -111,7 +111,7 @@ class RepositoryTestCase(RepositoryTestCaseBase):
         """
         self.repository.put(H(0), b'foo')
         self.assert_equal(self.repository.get(H(0)), b'foo')
-        self.repository.commit()
+        self.repository.commit(compact=False)
         self.repository.put(H(0), b'foo2')
         self.assert_equal(self.repository.get(H(0)), b'foo2')
         self.repository.rollback()
@@ -122,29 +122,29 @@ class RepositoryTestCase(RepositoryTestCaseBase):
         """
         self.repository.put(H(0), b'foo')
         self.repository.put(H(0), b'foo2')
-        self.repository.commit()
+        self.repository.commit(compact=False)
         self.assert_equal(self.repository.get(H(0)), b'foo2')
 
     def test_single_kind_transactions(self):
         # put
         self.repository.put(H(0), b'foo')
-        self.repository.commit()
+        self.repository.commit(compact=False)
         self.repository.close()
         # replace
         self.repository = self.open()
         with self.repository:
             self.repository.put(H(0), b'bar')
-            self.repository.commit()
+            self.repository.commit(compact=False)
         # delete
         self.repository = self.open()
         with self.repository:
             self.repository.delete(H(0))
-            self.repository.commit()
+            self.repository.commit(compact=False)
 
     def test_list(self):
         for x in range(100):
             self.repository.put(H(x), b'SOMEDATA')
-        self.repository.commit()
+        self.repository.commit(compact=False)
         all = self.repository.list()
         self.assert_equal(len(all), 100)
         first_half = self.repository.list(limit=50)
@@ -158,7 +158,7 @@ class RepositoryTestCase(RepositoryTestCaseBase):
     def test_scan(self):
         for x in range(100):
             self.repository.put(H(x), b'SOMEDATA')
-        self.repository.commit()
+        self.repository.commit(compact=False)
         all = self.repository.scan()
         assert len(all) == 100
         first_half = self.repository.scan(limit=50)
@@ -186,6 +186,8 @@ class LocalRepositoryTestCase(RepositoryTestCaseBase):
     def _assert_sparse(self):
         # The superseded 123456... PUT
         assert self.repository.compact[0] == 41 + 9
+        # a COMMIT
+        assert self.repository.compact[1] == 9
         # The DELETE issued by the superseding PUT (or issued directly)
         assert self.repository.compact[2] == 41
         self.repository._rebuild_sparse(0)
@@ -194,14 +196,14 @@ class LocalRepositoryTestCase(RepositoryTestCaseBase):
     def test_sparse1(self):
         self.repository.put(H(0), b'foo')
         self.repository.put(H(1), b'123456789')
-        self.repository.commit()
+        self.repository.commit(compact=False)
         self.repository.put(H(1), b'bar')
         self._assert_sparse()
 
     def test_sparse2(self):
         self.repository.put(H(0), b'foo')
         self.repository.put(H(1), b'123456789')
-        self.repository.commit()
+        self.repository.commit(compact=False)
         self.repository.delete(H(1))
         self._assert_sparse()
 
@@ -216,14 +218,14 @@ class LocalRepositoryTestCase(RepositoryTestCaseBase):
         # ...while _rebuild_sparse can mark whole segments as completely sparse (which then includes the segment magic)
         assert self.repository.compact[0] == 41 + 41 + 4 + len(MAGIC)
 
-        self.repository.commit()
+        self.repository.commit(compact=True)
         assert 0 not in [segment for segment, _ in self.repository.io.segment_iterator()]
 
     def test_uncommitted_garbage(self):
         # uncommitted garbage should be no problem, it is cleaned up automatically.
         # we just have to be careful with invalidation of cached FDs in LoggedIO.
         self.repository.put(H(0), b'foo')
-        self.repository.commit()
+        self.repository.commit(compact=False)
         # write some crap to a uncommitted segment file
         last_segment = self.repository.io.get_latest_segment()
         with open(self.repository.io.segment_filename(last_segment + 1), 'wb') as f:
@@ -233,7 +235,7 @@ class LocalRepositoryTestCase(RepositoryTestCaseBase):
         self.repository = self.open()
         with self.repository:
             self.repository.put(H(0), b'bar')  # this may trigger compact_segments()
-            self.repository.commit()
+            self.repository.commit(compact=True)
         # the point here is that nothing blows up with an exception.
 
 
@@ -245,33 +247,30 @@ class RepositoryCommitTestCase(RepositoryTestCaseBase):
             if name.startswith('index.'):
                 os.unlink(os.path.join(self.repository.path, name))
         self.reopen()
-        with self.repository:
-            self.assert_equal(len(self.repository), 3)
-            self.assert_equal(self.repository.check(), True)
+        self.assert_equal(len(self.repository), 3)
+        self.assert_equal(self.repository.check(), True)
 
     def test_crash_before_compact_segments(self):
         self.add_keys()
         self.repository.compact_segments = None
         try:
-            self.repository.commit()
+            self.repository.commit(compact=True)
         except TypeError:
             pass
         self.reopen()
-        with self.repository:
-            self.assert_equal(len(self.repository), 3)
-            self.assert_equal(self.repository.check(), True)
+        self.assert_equal(len(self.repository), 3)
+        self.assert_equal(self.repository.check(), True)
 
     def test_crash_before_write_index(self):
         self.add_keys()
         self.repository.write_index = None
         try:
-            self.repository.commit()
+            self.repository.commit(compact=False)
         except TypeError:
             pass
         self.reopen()
-        with self.repository:
-            self.assert_equal(len(self.repository), 3)
-            self.assert_equal(self.repository.check(), True)
+        self.assert_equal(len(self.repository), 3)
+        self.assert_equal(self.repository.check(), True)
 
     def test_replay_lock_upgrade_old(self):
         self.add_keys()
@@ -303,29 +302,27 @@ class RepositoryCommitTestCase(RepositoryTestCaseBase):
         self.add_keys()
         self.repository.io.delete_segment = None
         try:
-            self.repository.commit()
+            self.repository.commit(compact=False)
         except TypeError:
             pass
         self.reopen()
-        with self.repository:
-            self.assert_equal(len(self.repository), 3)
-            self.assert_equal(self.repository.check(), True)
-            self.assert_equal(len(self.repository), 3)
+        self.assert_equal(len(self.repository), 3)
+        self.assert_equal(self.repository.check(), True)
+        self.assert_equal(len(self.repository), 3)
 
     def test_ignores_commit_tag_in_data(self):
         self.repository.put(H(0), LoggedIO.COMMIT)
         self.reopen()
-        with self.repository:
-            io = self.repository.io
-            assert not io.is_committed_segment(io.get_latest_segment())
+        io = self.repository.io
+        assert not io.is_committed_segment(io.get_latest_segment())
 
     def test_moved_deletes_are_tracked(self):
         self.repository.put(H(1), b'1')
         self.repository.put(H(2), b'2')
-        self.repository.commit()
-        self.repo_dump('p1 p2 cc')
+        self.repository.commit(compact=False)
+        self.repo_dump('p1 p2 c')
         self.repository.delete(H(1))
-        self.repository.commit()
+        self.repository.commit(compact=True)
         self.repo_dump('d1 cc')
         last_segment = self.repository.io.get_latest_segment() - 1
         num_deletes = 0
@@ -336,7 +333,7 @@ class RepositoryCommitTestCase(RepositoryTestCaseBase):
         assert num_deletes == 1
         assert last_segment in self.repository.compact
         self.repository.put(H(3), b'3')
-        self.repository.commit()
+        self.repository.commit(compact=True)
         self.repo_dump('p3 cc')
         assert last_segment not in self.repository.compact
         assert not self.repository.io.segment_exists(last_segment)
@@ -353,7 +350,7 @@ class RepositoryCommitTestCase(RepositoryTestCaseBase):
         self.repository.put(H(1), b'1')
         # This is the segment with our original PUT of interest
         put_segment = get_latest_segment()
-        self.repository.commit()
+        self.repository.commit(compact=False)
 
         # We now delete H(1), and force this segment to not be compacted, which can happen
         # if it's not sparse enough (symbolized by H(2) here).
@@ -365,12 +362,12 @@ class RepositoryCommitTestCase(RepositoryTestCaseBase):
         del self.repository.compact[put_segment]
         del self.repository.compact[delete_segment]
 
-        self.repository.commit()
+        self.repository.commit(compact=True)
 
         # Now we perform an unrelated operation on the segment containing the DELETE,
         # causing it to be compacted.
         self.repository.delete(H(2))
-        self.repository.commit()
+        self.repository.commit(compact=True)
 
         assert self.repository.io.segment_exists(put_segment)
         assert not self.repository.io.segment_exists(delete_segment)
@@ -386,7 +383,7 @@ class RepositoryCommitTestCase(RepositoryTestCaseBase):
         self.repository.put(H(1), b'1')
         self.repository.delete(H(1))
         assert self.repository.shadow_index[H(1)] == [0]
-        self.repository.commit()
+        self.repository.commit(compact=True)
         self.repo_dump('p1 d1 cc')
         # note how an empty list means that nothing is shadowed for sure
         assert self.repository.shadow_index[H(1)] == []  # because the delete is considered unstable
@@ -416,21 +413,21 @@ class RepositoryAppendOnlyTestCase(RepositoryTestCaseBase):
         def segments_in_repository():
             return len(list(self.repository.io.segment_iterator()))
         self.repository.put(H(0), b'foo')
-        self.repository.commit()
+        self.repository.commit(compact=False)
 
         self.repository.append_only = False
         assert segments_in_repository() == 2
         self.repository.put(H(0), b'foo')
-        self.repository.commit()
+        self.repository.commit(compact=True)
         # normal: compact squashes the data together, only one segment
-        assert segments_in_repository() == 4
+        assert segments_in_repository() == 2
 
         self.repository.append_only = True
-        assert segments_in_repository() == 4
+        assert segments_in_repository() == 2
         self.repository.put(H(0), b'foo')
-        self.repository.commit()
+        self.repository.commit(compact=False)
         # append only: does not compact, only new segments written
-        assert segments_in_repository() == 6
+        assert segments_in_repository() == 4
 
 
 class RepositoryFreeSpaceTestCase(RepositoryTestCaseBase):
@@ -439,11 +436,9 @@ class RepositoryFreeSpaceTestCase(RepositoryTestCaseBase):
         self.repository.config.set('repository', 'additional_free_space', '1000T')
         self.repository.save_key(b'shortcut to save_config')
         self.reopen()
-
-        with self.repository:
-            self.repository.put(H(0), b'foobar')
-            with pytest.raises(Repository.InsufficientFreeSpaceError):
-                self.repository.commit()
+        self.repository.put(H(0), b'foobar')
+        with pytest.raises(Repository.InsufficientFreeSpaceError):
+            self.repository.commit(compact=False)
         assert os.path.exists(self.repository.path)
 
     def test_create_free_space(self):
@@ -462,37 +457,35 @@ class QuotaTestCase(RepositoryTestCaseBase):
         assert self.repository.storage_quota_use == 1234 + 5678 + 2 * 41
         self.repository.delete(H(1))
         assert self.repository.storage_quota_use == 1234 + 5678 + 2 * 41  # we have not compacted yet
-        self.repository.commit()
-        assert self.repository.storage_quota_use == 5678 + 41
+        self.repository.commit(compact=False)
+        assert self.repository.storage_quota_use == 1234 + 5678 + 2 * 41  # we have not compacted yet
         self.reopen()
-        with self.repository:
-            # Open new transaction; hints and thus quota data is not loaded unless needed.
-            self.repository.put(H(3), b'')
-            self.repository.delete(H(3))
-            assert self.repository.storage_quota_use == 5678 + 2 * 41  # we have not compacted yet
-            self.repository.commit()
-            assert self.repository.storage_quota_use == 5678 + 41
+        # Open new transaction; hints and thus quota data is not loaded unless needed.
+        self.repository.put(H(3), b'')
+        self.repository.delete(H(3))
+        assert self.repository.storage_quota_use == 1234 + 5678 + 3 * 41  # we have not compacted yet
+        self.repository.commit(compact=True)
+        assert self.repository.storage_quota_use == 5678 + 41
 
     def test_exceed_quota(self):
         assert self.repository.storage_quota_use == 0
         self.repository.storage_quota = 50
         self.repository.put(H(1), b'')
         assert self.repository.storage_quota_use == 41
-        self.repository.commit()
+        self.repository.commit(compact=False)
         with pytest.raises(Repository.StorageQuotaExceeded):
             self.repository.put(H(2), b'')
         assert self.repository.storage_quota_use == 82
         with pytest.raises(Repository.StorageQuotaExceeded):
-            self.repository.commit()
+            self.repository.commit(compact=False)
         assert self.repository.storage_quota_use == 82
         self.reopen()
-        with self.repository:
-            self.repository.storage_quota = 100
-            # Open new transaction; hints and thus quota data is not loaded unless needed.
-            self.repository.put(H(1), b'')
-            assert self.repository.storage_quota_use == 82  # we have 2 puts for H(1) here and not yet compacted.
-            self.repository.commit()
-            assert self.repository.storage_quota_use == 41  # now we have compacted.
+        self.repository.storage_quota = 100
+        # Open new transaction; hints and thus quota data is not loaded unless needed.
+        self.repository.put(H(1), b'')
+        assert self.repository.storage_quota_use == 82  # we have 2 puts for H(1) here and not yet compacted.
+        self.repository.commit(compact=True)
+        assert self.repository.storage_quota_use == 41  # now we have compacted.
 
 
 class NonceReservation(RepositoryTestCaseBase):
@@ -526,14 +519,14 @@ class NonceReservation(RepositoryTestCaseBase):
                 self.repository.commit_nonce_reservation(0x200, 15)
 
             self.repository.commit_nonce_reservation(0x200, None)
-            with open(os.path.join(self.repository.path, "nonce"), "r") as fd:
+            with open(os.path.join(self.repository.path, "nonce")) as fd:
                 assert fd.read() == "0000000000000200"
 
             with pytest.raises(Exception):
                 self.repository.commit_nonce_reservation(0x200, 15)
 
             self.repository.commit_nonce_reservation(0x400, 0x200)
-            with open(os.path.join(self.repository.path, "nonce"), "r") as fd:
+            with open(os.path.join(self.repository.path, "nonce")) as fd:
                 assert fd.read() == "0000000000000400"
 
 
@@ -541,13 +534,13 @@ class RepositoryAuxiliaryCorruptionTestCase(RepositoryTestCaseBase):
     def setUp(self):
         super().setUp()
         self.repository.put(H(0), b'foo')
-        self.repository.commit()
+        self.repository.commit(compact=False)
         self.repository.close()
 
     def do_commit(self):
         with self.repository:
             self.repository.put(H(0), b'fox')
-            self.repository.commit()
+            self.repository.commit(compact=False)
 
     def test_corrupted_hints(self):
         with open(os.path.join(self.repository.path, 'hints.1'), 'ab') as fd:
@@ -644,9 +637,9 @@ class RepositoryAuxiliaryCorruptionTestCase(RepositoryTestCaseBase):
             assert self.repository.get(H(0)) == b'foo'
             self.repository.put(H(1), b'bar')
             self.repository.put(H(2), b'baz')
-            self.repository.commit()
+            self.repository.commit(compact=False)
             self.repository.put(H(2), b'bazz')
-            self.repository.commit()
+            self.repository.commit(compact=False)
 
         hints_path = os.path.join(self.repository.path, 'hints.5')
         with open(hints_path, 'r+b') as fd:
@@ -664,7 +657,7 @@ class RepositoryAuxiliaryCorruptionTestCase(RepositoryTestCaseBase):
             self.repository.append_only = False
             self.repository.put(H(3), b'1234')
             # Do a compaction run. Succeeds, since the failed checksum prompted a rebuild of the index+hints.
-            self.repository.commit()
+            self.repository.commit(compact=True)
 
             assert len(self.repository) == 4
             assert self.repository.get(H(0)) == b'foo'
@@ -680,7 +673,7 @@ class RepositoryAuxiliaryCorruptionTestCase(RepositoryTestCaseBase):
             self.repository.put(H(3), b'1234')
             # Do a compaction run. Fails, since the corrupted refcount was not detected and leads to an assertion failure.
             with pytest.raises(AssertionError) as exc_info:
-                self.repository.commit()
+                self.repository.commit(compact=True)
             assert 'Corrupted segment reference count' in str(exc_info.value)
 
 
@@ -702,13 +695,13 @@ class RepositoryCheckTestCase(RepositoryTestCaseBase):
         for ids in segments:
             for id_ in ids:
                 self.repository.put(H(id_), b'data')
-            self.repository.commit()
+            self.repository.commit(compact=False)
 
     def get_head(self):
         return sorted(int(n) for n in os.listdir(os.path.join(self.tmppath, 'repository', 'data', '0')) if n.isdigit())[-1]
 
     def open_index(self):
-        return NSIndex.read(os.path.join(self.tmppath, 'repository', 'index.{}'.format(self.get_head())))
+        return NSIndex.read(os.path.join(self.tmppath, 'repository', f'index.{self.get_head()}'))
 
     def corrupt_object(self, id_):
         idx = self.open_index()
@@ -718,21 +711,21 @@ class RepositoryCheckTestCase(RepositoryTestCaseBase):
             fd.write(b'BOOM')
 
     def delete_segment(self, segment):
-        os.unlink(os.path.join(self.tmppath, 'repository', 'data', '0', str(segment)))
+        self.repository.io.delete_segment(segment)
 
     def delete_index(self):
-        os.unlink(os.path.join(self.tmppath, 'repository', 'index.{}'.format(self.get_head())))
+        os.unlink(os.path.join(self.tmppath, 'repository', f'index.{self.get_head()}'))
 
     def rename_index(self, new_name):
-        os.rename(os.path.join(self.tmppath, 'repository', 'index.{}'.format(self.get_head())),
+        os.rename(os.path.join(self.tmppath, 'repository', f'index.{self.get_head()}'),
                   os.path.join(self.tmppath, 'repository', new_name))
 
     def list_objects(self):
-        return set(int(key) for key in self.repository.list())
+        return {int(key) for key in self.repository.list()}
 
     def test_repair_corrupted_segment(self):
         self.add_objects([[1, 2, 3], [4, 5], [6]])
-        self.assert_equal(set([1, 2, 3, 4, 5, 6]), self.list_objects())
+        self.assert_equal({1, 2, 3, 4, 5, 6}, self.list_objects())
         self.check(status=True)
         self.corrupt_object(5)
         self.assert_raises(IntegrityError, lambda: self.get_objects(5))
@@ -744,22 +737,22 @@ class RepositoryCheckTestCase(RepositoryTestCaseBase):
         self.check(repair=True, status=True)
         self.get_objects(4)
         self.check(status=True)
-        self.assert_equal(set([1, 2, 3, 4, 6]), self.list_objects())
+        self.assert_equal({1, 2, 3, 4, 6}, self.list_objects())
 
     def test_repair_missing_segment(self):
         self.add_objects([[1, 2, 3], [4, 5, 6]])
-        self.assert_equal(set([1, 2, 3, 4, 5, 6]), self.list_objects())
+        self.assert_equal({1, 2, 3, 4, 5, 6}, self.list_objects())
         self.check(status=True)
         self.delete_segment(2)
         self.repository.rollback()
         self.check(repair=True, status=True)
-        self.assert_equal(set([1, 2, 3]), self.list_objects())
+        self.assert_equal({1, 2, 3}, self.list_objects())
 
     def test_repair_missing_commit_segment(self):
         self.add_objects([[1, 2, 3], [4, 5, 6]])
         self.delete_segment(3)
         self.assert_raises(Repository.ObjectNotFound, lambda: self.get_objects(4))
-        self.assert_equal(set([1, 2, 3]), self.list_objects())
+        self.assert_equal({1, 2, 3}, self.list_objects())
 
     def test_repair_corrupted_commit_segment(self):
         self.add_objects([[1, 2, 3], [4, 5, 6]])
@@ -769,7 +762,7 @@ class RepositoryCheckTestCase(RepositoryTestCaseBase):
         self.assert_raises(Repository.ObjectNotFound, lambda: self.get_objects(4))
         self.check(status=True)
         self.get_objects(3)
-        self.assert_equal(set([1, 2, 3]), self.list_objects())
+        self.assert_equal({1, 2, 3}, self.list_objects())
 
     def test_repair_no_commits(self):
         self.add_objects([[1, 2, 3]])
@@ -781,17 +774,17 @@ class RepositoryCheckTestCase(RepositoryTestCaseBase):
         self.check(status=False)
         self.assert_equal(self.list_indices(), ['index.1'])
         self.check(repair=True, status=True)
-        self.assert_equal(self.list_indices(), ['index.3'])
+        self.assert_equal(self.list_indices(), ['index.2'])
         self.check(status=True)
         self.get_objects(3)
-        self.assert_equal(set([1, 2, 3]), self.list_objects())
+        self.assert_equal({1, 2, 3}, self.list_objects())
 
     def test_repair_missing_index(self):
         self.add_objects([[1, 2, 3], [4, 5, 6]])
         self.delete_index()
         self.check(status=True)
         self.get_objects(4)
-        self.assert_equal(set([1, 2, 3, 4, 5, 6]), self.list_objects())
+        self.assert_equal({1, 2, 3, 4, 5, 6}, self.list_objects())
 
     def test_repair_index_too_new(self):
         self.add_objects([[1, 2, 3], [4, 5, 6]])
@@ -800,33 +793,52 @@ class RepositoryCheckTestCase(RepositoryTestCaseBase):
         self.check(status=True)
         self.assert_equal(self.list_indices(), ['index.3'])
         self.get_objects(4)
-        self.assert_equal(set([1, 2, 3, 4, 5, 6]), self.list_objects())
+        self.assert_equal({1, 2, 3, 4, 5, 6}, self.list_objects())
 
     def test_crash_before_compact(self):
         self.repository.put(H(0), b'data')
         self.repository.put(H(0), b'data2')
         # Simulate a crash before compact
         with patch.object(Repository, 'compact_segments') as compact:
-            self.repository.commit()
-            compact.assert_called_once_with()
+            self.repository.commit(compact=True)
+            compact.assert_called_once_with(0.1)
         self.reopen()
-        with self.repository:
-            self.check(repair=True)
-            self.assert_equal(self.repository.get(H(0)), b'data2')
+        self.check(repair=True)
+        self.assert_equal(self.repository.get(H(0)), b'data2')
 
 
 class RepositoryHintsTestCase(RepositoryTestCaseBase):
 
+    def test_hints_persistence(self):
+        self.repository.put(H(0), b'data')
+        self.repository.delete(H(0))
+        self.repository.commit(compact=False)
+        shadow_index_expected = self.repository.shadow_index
+        compact_expected = self.repository.compact
+        segments_expected = self.repository.segments
+        # close and re-open the repository (create fresh Repository instance) to
+        # check whether hints were persisted to / reloaded from disk
+        self.reopen()
+        # see also do_compact()
+        self.repository.put(H(42), b'foobar')  # this will call prepare_txn() and load the hints data
+        # check if hints persistence worked:
+        self.assert_equal(shadow_index_expected, self.repository.shadow_index)
+        self.assert_equal(compact_expected, self.repository.compact)
+        del self.repository.segments[2]  # ignore the segment created by put(H(42), ...)
+        self.assert_equal(segments_expected, self.repository.segments)
+
     def test_hints_behaviour(self):
         self.repository.put(H(0), b'data')
         self.assert_equal(self.repository.shadow_index, {})
         assert len(self.repository.compact) == 0
         self.repository.delete(H(0))
-        self.repository.commit()
-        # this is to make the previous delete "stable", see delete_is_not_stable:
-        self.repository.put(H(1), b'data')
-        self.repository.commit()
-        # in 1.1-maint, commit() always implicitly compacts.
+        self.repository.commit(compact=False)
+        # now there should be an entry for H(0) in shadow_index
+        self.assert_in(H(0), self.repository.shadow_index)
+        self.assert_equal(len(self.repository.shadow_index[H(0)]), 1)
+        self.assert_in(0, self.repository.compact)  # segment 0 can be compacted
+        self.repository.put(H(42), b'foobar')  # see also do_compact()
+        self.repository.commit(compact=True, threshold=0.0)  # compact completely!
         # nothing to compact any more! no info left about stuff that does not exist any more:
         self.assert_not_in(H(0), self.repository.shadow_index)
         # segment 0 was compacted away, no info about it left:
@@ -927,17 +939,17 @@ class RemoteRepositoryTestCase(RepositoryTestCase):
         # XXX without next line we get spurious test fails when using pytest-xdist, root cause unknown:
         logging.getLogger().setLevel(logging.INFO)
         # note: test logger is on info log level, so --info gets added automagically
-        assert self.repository.borg_cmd(args, testing=False) == ['borg', 'serve', '--umask=077', '--info']
+        assert self.repository.borg_cmd(args, testing=False) == ['borg', 'serve', '--info']
         args.remote_path = 'borg-0.28.2'
-        assert self.repository.borg_cmd(args, testing=False) == ['borg-0.28.2', 'serve', '--umask=077', '--info']
+        assert self.repository.borg_cmd(args, testing=False) == ['borg-0.28.2', 'serve', '--info']
         args.debug_topics = ['something_client_side', 'repository_compaction']
-        assert self.repository.borg_cmd(args, testing=False) == ['borg-0.28.2', 'serve', '--umask=077', '--info',
+        assert self.repository.borg_cmd(args, testing=False) == ['borg-0.28.2', 'serve', '--info',
                                                                  '--debug-topic=borg.debug.repository_compaction']
         args = self._get_mock_args()
         args.storage_quota = 0
-        assert self.repository.borg_cmd(args, testing=False) == ['borg', 'serve', '--umask=077', '--info']
+        assert self.repository.borg_cmd(args, testing=False) == ['borg', 'serve', '--info']
         args.storage_quota = 314159265
-        assert self.repository.borg_cmd(args, testing=False) == ['borg', 'serve', '--umask=077', '--info',
+        assert self.repository.borg_cmd(args, testing=False) == ['borg', 'serve', '--info',
                                                                  '--storage-quota=314159265']
         args.rsh = 'ssh -i foo'
         self.repository._args = args
@@ -955,18 +967,18 @@ class RemoteLegacyFree(RepositoryTestCaseBase):
     def test_legacy_free(self):
         # put
         self.repository.put(H(0), b'foo')
-        self.repository.commit()
+        self.repository.commit(compact=False)
         self.repository.close()
         # replace
         self.repository = self.open()
         with self.repository:
             self.repository.put(H(0), b'bar')
-            self.repository.commit()
+            self.repository.commit(compact=False)
         # delete
         self.repository = self.open()
         with self.repository:
             self.repository.delete(H(0))
-            self.repository.commit()
+            self.repository.commit(compact=False)
 
 
 class RemoteRepositoryCheckTestCase(RepositoryCheckTestCase):
@@ -979,6 +991,14 @@ class RemoteRepositoryCheckTestCase(RepositoryCheckTestCase):
         # skip this test, we can't mock-patch a Repository class in another process!
         pass
 
+    def test_repair_missing_commit_segment(self):
+        # skip this test, files in RemoteRepository cannot be deleted
+        pass
+
+    def test_repair_missing_segment(self):
+        # skip this test, files in RemoteRepository cannot be deleted
+        pass
+
 
 class RemoteLoggerTestCase(BaseTestCase):
     def setUp(self):
